[data-md-color-scheme="default"] {
  --md-accent-fg-color: #1D062C;
  --md-primary-fg-color: #C77A8A;
}
[data-md-color-scheme="slate"] {
  --md-accent-fg-color: #1D062C;
  --md-primary-fg-color: #C77A8A;
}
